import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["isOpen", "target", "onOpen", "onClose", "onToggle", "onClick", "hasCloseOnOutsideClick", "hasCloseOnEsc"],
  _excluded2 = ["as", "children", "isOpen", "className"];
/* eslint-disable react/prop-types */
import { forwardRef, useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { createPortal, findDOMNode } from 'react-dom';
import { isFragment } from 'react-is';
import cx from 'classnames';
import PropTypes from 'prop-types';
import useSSR from 'use-ssr';
import useMergeRefs from '@s-ui/react-hooks/lib/useMergeRefs';
import useMount from '@s-ui/react-hooks/lib/useMount';
import { BASE_CLASS, DEFAULT_IS_OPEN } from './settings.js';
import { jsx as _jsx } from "react/jsx-runtime";
var usePortal = function usePortal(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
    _ref$isOpen = _ref.isOpen,
    defaultIsOpen = _ref$isOpen === void 0 ? DEFAULT_IS_OPEN : _ref$isOpen,
    target = _ref.target,
    onOpen = _ref.onOpen,
    onClose = _ref.onClose,
    onToggle = _ref.onToggle,
    onClick = _ref.onClick,
    _ref$hasCloseOnOutsid = _ref.hasCloseOnOutsideClick,
    hasCloseOnOutsideClick = _ref$hasCloseOnOutsid === void 0 ? false : _ref$hasCloseOnOutsid,
    _ref$hasCloseOnEsc = _ref.hasCloseOnEsc,
    hasCloseOnEsc = _ref$hasCloseOnEsc === void 0 ? false : _ref$hasCloseOnEsc,
    eventHandlers = _objectWithoutPropertiesLoose(_ref, _excluded);
  var _useSSR = useSSR(),
    isServer = _useSSR.isServer;
  var _useState = useState(),
    isReady = _useState[0],
    setIsReady = _useState[1];
  var _useState2 = useState(defaultIsOpen),
    isOpened = _useState2[0],
    setIsOpened = _useState2[1];
  var triggerElement = useRef(); // this is the element you are clicking/hovering/whatever, to trigger opening the portal
  var portal = useRef();
  useMount(function () {
    setIsReady(true);
  });
  var elToMountTo = useMemo(function () {
    if (isServer) return;
    return target && findDOMNode(target) || document.body;
  }, [isServer, target]);
  var createCustomEvent = function createCustomEvent(event) {
    if (!event) return {
      portal: portal,
      triggerElement: triggerElement,
      event: event
    };
    var response = event || {};
    if (response.persist) response.persist();
    response.portal = portal;
    response.triggerElement = triggerElement;
    response.event = event;
    return response;
  };

  // this should handle all eventHandlers like onClick, onMouseOver, etc. passed into the config
  var customEventHandlers = Object.entries(eventHandlers).reduce(function (acc, _ref2) {
    var handlerName = _ref2[0],
      eventHandler = _ref2[1];
    acc[handlerName] = function (event) {
      if (isServer) return;
      eventHandler(createCustomEvent(event));
    };
    return acc;
  }, {});
  var setIsOpen = useCallback(function (value) {
    return function (event) {
      if (isServer) return;
      var customEvent = createCustomEvent(event);
      if (onOpen && value === true) setTimeout(function () {
        return onOpen(customEvent);
      }, 0);
      if (onClose && value === false) onClose(customEvent);
      setIsOpened(value);
    };
  }, [isServer, setIsOpened, onClose, onOpen]);
  var openPortal = setIsOpen(true);
  var closePortal = setIsOpen(false);
  var togglePortal = function togglePortal(event) {
    return isOpened ? closePortal(event) : openPortal(event);
  };
  var handleKeydown = useCallback(function (event) {
    return event.key === 'Escape' && hasCloseOnEsc ? closePortal(event) : undefined;
  }, [hasCloseOnEsc, closePortal]);
  var handleOutsideMouseClick = useCallback(function (event) {
    var containsTarget = function containsTarget(target) {
      var _target$current;
      return (_target$current = target.current) == null ? void 0 : _target$current.contains(event.target);
    };
    if (containsTarget(portal) || event.button !== 0 || !isOpened || containsTarget(triggerElement)) {
      return;
    }
    if (hasCloseOnOutsideClick) {
      closePortal(event);
    }
  }, [closePortal, hasCloseOnOutsideClick, portal, isOpened]);
  var handleMouseDown = useCallback(function (event) {
    if (isServer || !(portal.current instanceof HTMLElement)) return;
    var customEvent = createCustomEvent(event);
    if (portal.current.contains(customEvent.target) && onClick) onClick(customEvent);
    handleOutsideMouseClick(event);
  }, [handleOutsideMouseClick, isServer, onClick]);

  // used to remove the event listeners on unmount
  var eventListeners = useRef({});
  useEffect(function () {
    if (isServer) return;
    if (!(elToMountTo instanceof HTMLElement) || !(portal.current instanceof HTMLElement)) {
      return;
    }

    // TODO: eventually will need to figure out a better solution for this.
    // Surely we can find a way to map onScroll/onWheel -> scroll/wheel better,
    // but for all other event handlers. For now this works.
    var eventHandlerMap = {
      onScroll: 'scroll',
      onWheel: 'wheel'
    };
    Object.entries(eventHandlerMap).forEach(function (_ref3) {
      var handlerName /* onScroll */ = _ref3[0],
        eventListenerName /* scroll */ = _ref3[1];
      if (!eventHandlers[handlerName]) return;
      eventListeners.current[handlerName] = function (event) {
        return eventHandlers[handlerName](createCustomEvent(event));
      };
      document.addEventListener(eventListenerName, eventListeners.current[handlerName]);
    });
    document.addEventListener('keydown', handleKeydown);
    document.addEventListener('mousedown', handleMouseDown);
    return function () {
      // handles all special case handlers. Currently only onScroll and onWheel
      Object.entries(eventHandlerMap).forEach(function (_ref4) {
        var handlerName = _ref4[0],
          eventListenerName = _ref4[1];
        if (!eventHandlers[handlerName]) return;
        document.removeEventListener(eventListenerName, eventListeners.current[handlerName]);
        delete eventListeners.current[handlerName]; // eslint-disable-line react-hooks/exhaustive-deps
      });

      document.removeEventListener('keydown', handleKeydown);
      document.removeEventListener('mousedown', handleMouseDown);
    };
  }, [isServer, handleOutsideMouseClick, handleKeydown, elToMountTo, portal, eventHandlers, handleMouseDown]);
  var Portal = /*#__PURE__*/forwardRef(function (_ref5, forwardedRef) {
    var _ref5$as = _ref5.as,
      As = _ref5$as === void 0 ? 'div' : _ref5$as,
      children = _ref5.children,
      isOpenProp = _ref5.isOpen,
      className = _ref5.className,
      props = _objectWithoutPropertiesLoose(_ref5, _excluded2);
    var ref = useMergeRefs(forwardedRef, portal);
    useEffect(function () {
      if (isServer) return;
      if (isOpenProp !== undefined) {
        setIsOpened(isOpenProp);
      }
    }, [isOpenProp]);
    return isReady && isOpened ? /*#__PURE__*/createPortal( /*#__PURE__*/_jsx(As, _extends({}, !isFragment( /*#__PURE__*/_jsx(As, {})) && _extends({
      ref: ref,
      className: cx(BASE_CLASS, className)
    }, props), {
      children: children
    })), target || document.body) : null;
  });
  Portal.displayName = 'Portal';
  return Object.assign([Portal, openPortal, closePortal, isOpened, togglePortal, triggerElement, portal], _extends({
    isOpen: isOpened,
    triggerRef: triggerElement,
    open: openPortal,
    close: closePortal,
    toggle: togglePortal,
    Portal: Portal,
    portalRef: portal
  }, customEventHandlers, {
    bind: _extends({
      // used if you want to spread all html attributes onto the target element
      ref: triggerElement
    }, customEventHandlers)
  }));
};
export default usePortal;